﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

using Newtonsoft.Json;
using ProtoBuf;

using NetworkCommsDotNet.DPSBase;
using NetworkCommsDotNet.Tools;

namespace TestProject
{
    /// <summary>
    /// 채팅 메시지
    /// </summary>
    [ProtoContract]
    [JsonObject(MemberSerialization.OptIn)]
    public class ChatMessage : IExplicitlySerialize
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 소스 식별자
        /// </summary>
        [ProtoMember(1)]
        [JsonProperty]
        private string sourceIdentifier;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 소스 식별자 - SourceIdentifier

        /// <summary>
        /// 소스 식별자
        /// </summary>
        public ShortGuid SourceIdentifier
        {
            get
            {
                return new ShortGuid(this.sourceIdentifier);
            }
        }

        #endregion
        #region 소스명 - SourceName

        /// <summary>
        /// 소스명
        /// </summary>
        [ProtoMember(2)]
        [JsonProperty]
        public string SourceName { get; private set; }

        #endregion
        #region 메시지 - Message

        /// <summary>
        /// 메시지
        /// </summary>
        [ProtoMember(3)]
        [JsonProperty]
        public string Message { get; private set; }

        #endregion
        #region 메시지 인덱스 - MessageIndex

        /// <summary>
        /// 메시지 인덱스
        /// </summary>
        [ProtoMember(4)]
        [JsonProperty]
        public long MessageIndex { get; private set; }

        #endregion
        #region 릴레이 카운트 - RelayCount

        /// <summary>
        /// 릴레이 카운트
        /// </summary>
        [ProtoMember(5)]
        [JsonProperty]
        public int RelayCount { get; private set; }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 생성자 - ChatMessage()

        /// <summary>
        /// 생성자
        /// </summary>
        private ChatMessage()
        {
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ChatMessage(sourceIdentifier, sourceName, message, messageIndex)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="sourceIdentifier">소스 식별자</param>
        /// <param name="sourceName">소스명</param>
        /// <param name="message">메시지</param>
        /// <param name="messageIndex">메시지 인덱스</param>
        public ChatMessage(ShortGuid sourceIdentifier, string sourceName, string message, long messageIndex)
        {
            this.sourceIdentifier = sourceIdentifier;

            SourceName   = sourceName;
            Message      = message;
            MessageIndex = messageIndex;
            RelayCount   = 0;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 역직렬화하기 - Deserialize(inputStream, chatMessage)

        /// <summary>
        /// 역직렬화하기
        /// </summary>
        /// <param name="inputStream">입력 스트림</param>
        /// <param name="chatMessage">채팅 메시지</param>
        public static void Deserialize(Stream inputStream, out ChatMessage chatMessage)
        {
            chatMessage = new ChatMessage();

            chatMessage.Deserialize(inputStream);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 릴레이 카운트 증가시키기 - IncrementRelayCount()

        /// <summary>
        /// 릴레이 카운트 증가시키기
        /// </summary>
        public void IncrementRelayCount()
        {
            RelayCount++;
        }

        #endregion
        #region 직렬화하기 - Serialize(outputStream)

        /// <summary>
        /// 직렬화 하기
        /// </summary>
        /// <param name="outputStream">출력 스트림</param>
        public void Serialize(Stream outputStream)
        {
            List<byte[]> byteArrayList = new List<byte[]>();

            byte[] sourceIdentifierByteArray       = Encoding.UTF8.GetBytes(this.sourceIdentifier);
            byte[] sourceIdentifierLengthByteArray = BitConverter.GetBytes(sourceIdentifierByteArray.Length);

            byteArrayList.Add(sourceIdentifierLengthByteArray);
            byteArrayList.Add(sourceIdentifierByteArray);

            byte[] sourceNameByteArray       = Encoding.UTF8.GetBytes(SourceName);
            byte[] sourceNameLengthByteArray = BitConverter.GetBytes(sourceNameByteArray.Length);

            byteArrayList.Add(sourceNameLengthByteArray);
            byteArrayList.Add(sourceNameByteArray);

            byte[] messageByteArray       = Encoding.UTF8.GetBytes(Message);
            byte[] messageLengthByteArray = BitConverter.GetBytes(messageByteArray.Length);

            byteArrayList.Add(messageLengthByteArray);
            byteArrayList.Add(messageByteArray);

            byte[] messageIndexByteArray = BitConverter.GetBytes(MessageIndex);

            byteArrayList.Add(messageIndexByteArray);

            byte[] relayCountByteArray = BitConverter.GetBytes(RelayCount);

            byteArrayList.Add(relayCountByteArray);

            foreach(byte[] byteArray in byteArrayList)
            {
                outputStream.Write(byteArray, 0, byteArray.Length);
            }
        }

        #endregion
        #region 역직렬화하기 - Deserialize(inputStream)

        /// <summary>
        /// 역직렬화하기
        /// </summary>
        /// <param name="inputStream">입력 스트림</param>
        public void Deserialize(Stream inputStream)
        {            
            byte[] sourceIdentifierLengthByteArray = new byte[sizeof(int)];
            
            inputStream.Read(sourceIdentifierLengthByteArray, 0, sizeof(int));


            byte[] sourceIdentifierByteArray = new byte[BitConverter.ToInt32(sourceIdentifierLengthByteArray, 0)];
            
            inputStream.Read(sourceIdentifierByteArray, 0, sourceIdentifierByteArray.Length);

            this.sourceIdentifier = new string(Encoding.UTF8.GetChars(sourceIdentifierByteArray));


            byte[] sourceNameLengthByteArray = new byte[sizeof(int)];
            
            inputStream.Read(sourceNameLengthByteArray, 0, sizeof(int));


            byte[] sourceNameByteArray = new byte[BitConverter.ToInt32(sourceNameLengthByteArray, 0)];
            
            inputStream.Read(sourceNameByteArray, 0, sourceNameByteArray.Length);

            SourceName = new String(Encoding.UTF8.GetChars(sourceNameByteArray));


            byte[] messageLengthByteArray = new byte[sizeof(int)];
            
            inputStream.Read(messageLengthByteArray, 0, sizeof(int));


            byte[] messageByteArray = new byte[BitConverter.ToInt32(messageLengthByteArray, 0)];
            
            inputStream.Read(messageByteArray, 0, messageByteArray.Length);

            Message = new string(Encoding.UTF8.GetChars(messageByteArray));


            byte[] messageIndexByteArray = new byte[sizeof(long)];
            
            inputStream.Read(messageIndexByteArray, 0, sizeof(long));

            MessageIndex = BitConverter.ToInt64(messageIndexByteArray, 0);


            byte[] relayCountByteArray = new byte[sizeof(int)];
            
            inputStream.Read(relayCountByteArray, 0, sizeof(int));

            RelayCount = BitConverter.ToInt32(relayCountByteArray, 0);
        }

        #endregion
    }
}